#ifndef __LINUX_GUI_H
#define __LINUX_GUI_H


typedef unsigned char U8;
typedef unsigned short U16;
typedef unsigned long U32;
typedef unsigned long long U64;

#define UNIT_WIDTH 8


typedef struct {
	short x;
	short y;
}PIXEL;

typedef struct {
	short left;
	short top;
	short right;
	short bottom;
}RECT;

typedef enum {
	Mode_SRC,
	Mode_NOT_SRC,
	Mode_SRC_OR_DST,
	Mode_SRC_AND_DST,
	Mode_SRC_XOR_DST,
	Mode_SRC_OR_NOT_DST,
	Mode_SRC_AND_NOT_DST,
	Mode_SRC_XOR_NOT_DST,
	Mode_NOT_SRC_OR_DST,
	Mode_NOT_SRC_AND_DST,
	Mode_NOT_SRC_XOR_DST,
	Mode_NOT_SRC_OR_NOT_DST,
	Mode_NOT_SRC_AND_NOT_DST,
	Mode_NOT_SRC_XOR_NOT_DST,
	InvalidMode
} CopyMode;

typedef struct {
	U16 height;
	U8 data[0];
} Pattern;

typedef enum {
	WhitePattern = 0,
	BlackPattern,
	DarkGreyPattern,
	LightGreyPattern,
	MicroPengPattern,
	HoriDotPattern,
	VertDotPattern,
	InvalidPattern
} PatternIndex;



typedef enum {
	GUI_WHITE =0,
	GUI_BLACK = 1
}Color;

typedef struct {
	Color color;
	CopyMode mode;
	PatternIndex fill_index;
} GraphicsContext;

typedef struct {
	U8 flag;
	short x,y,w,h,id;
}BackupBackGround;

typedef struct {
	U16 xoff;
	short yoff;
	U8 w;
	U8 h;
} GlyphInfo;


typedef struct {
	short width,height;
	void *img;
	char *caption;
}BITMAP;

typedef struct {
	short 	max,position,page_height;
	RECT 	first_r,second_r,third_r,last_r,button_r;
}SCROLLBAR;

typedef struct {
	short value;
	void *caption;
}CHECKBUTTON;

typedef struct {
	short counter_lines;	// the line counter of current textbox that can be 'visible'
	char string[256];
	short max_char;	// yes, the maximun charactor number of the textbox
	short min_empty_lines;	// minimun empty lines
	short start_line;	// let u set the start line freely
	short visible_lines;
}TEXTBOX;

typedef struct {
	short index;
	char str[32];
}ITEM;
	
typedef struct {
	short max_items;
	short selected;
	short drop_down_width,drop_down_height;
	ITEM *items;
}COMBOBOX;

typedef struct {
	short spacing;
	char str[32];
}SPINBUTTON;

typedef struct {
	char *str;
	void *icon;
	RECT r;		// this member is reserved for GUI internal functions...
}FORMCAPTION;

typedef struct {
	short max;
	short position;
}SLIDER;
/**********************************************
 *
 *	For each component,bit 7 
 *	is a 'Visible' flag
 *
 **********************************************/
#define COMPONENT_VISIBLE	0x01
#define COMPONENT_INVISIBLE	0x00

/**********************************************
 *
 *	Button component style
 *
 **********************************************/
#define BUTTON_NO_BORDER	1

/**********************************************
 *
 *	Bitmap component style
 *
 **********************************************/
#define BITMAP_BORDER			0x02	// draw a border around the bitmap
#define BITMAP_BUTTON			0x04	// use the bitmap as button
#define BITMAP_CAPTION_INNER	0x08	// the caption is shown in the bitmap , not out of
#define BITMAP_DOTBORDER		0x10	// draw a dot border around the bitmap
#define BITMAP_CAPTION_RIGHT	0x20
#define BITMAP_CAPTION_BOTTOM	0x40

/**********************************************
 *
 *	Form Frame Styles
 *
 **********************************************/
#define FORM_FRAME_CAPTION		0x01
#define FORM_FRAME_BORDER		0x02
#define FORM_SAVEBEHIND			0x04	// Auto save the background while create a new FORM
#define FORM_DLGFRAME				0x08

/**********************************************
 *
 *	Text alignment methods
 *
 **********************************************/
#define X_ALIGN_LEFT	0x01
#define X_ALIGN_MIDDLE	0x02
#define X_ALIGN_RIGHT	0x04
#define Y_ALIGN_TOP		0x08
#define Y_ALIGN_CENTER	0x10
#define Y_ALIGN_BOTTOM	0x20

/**********************************************
 *
 *		Label styles
 *
 **********************************************/
#define LABEL_BORDER	0x40
#define LABEL_MULTILINE	0x80

/**********************************************
 *
 *	The MENU status
 *
 **********************************************/
#define MENU_DROPPED	1
#define MENU_UNDROPPED	0

/**********************************************
 *
 *	SCROLLBAR styles
 *
 **********************************************/
#define VSCROLLBAR		0
#define HSCROLLBAR		1


/**********************************************
 *
 *	SPINBUTTON styles
 *
 **********************************************/
#define VSPINBUTTON		0
#define HSPINBUTTON		1

/**********************************************
 *
 *	COMBOBOX styles
 *
 **********************************************/
#define COMBOBOX_NO_BORDER		1
#define COMBOBOX_LEFT_ALIGN		2
#define COMBOBOX_RIGHT_ALIGN	4
#define COMBOBOX_NO_ARROW		8

/*********************************************
 *
 *	Indicator status
 *
 *********************************************/
#define INDICATOR_NORMAL	0x02
#define INDICATOR_SHIFT		0x04
#define INDICATOR_CAPSLOCK	0x08
#define INDICATOR_DOT	0x10

/**********************************************
 *
 *	Basic FORM elements definitions
 *
 **********************************************/
#define FORM_CAPTION		1
#define FORM_LABEL			2
#define FORM_BUTTON			3
#define FORM_BITMAP			4
#define FORM_MENU			5
#define FORM_MENUITEM		6
#define FORM_SYSTEM			7
#define FORM_SCROLLBAR		8
#define FORM_TEXTBOX		9
#define FORM_SPINBUTTON		10
#define FORM_CHECKBUTTON	11
#define FORM_COMBOBOX		12
#define FORM_SLIDER			13
#define FORM_INDICATOR	15
#define FORM_END			127

typedef struct {
	U8 type;
	U8 id;
	U16 x,y,w,h;		
	void *text;
	U8 style;		
	U8 status;
	U8 font;
}FORM;

/**********************************************************
 *
 *	I define the events of internal message queue here 
 *
 *		Mac Tseng 11/2/2000
 *
 **********************************************************/
#define EVENT_NOTHING		0
#define EVENT_CLICK 		1
#define EVENT_INPUTCHAR		2
#define EVENT_KEYDOWN		3
#define EVENT_KEYUP			4
#define EVENT_PEN_UP		5
#define EVENT_DESTROY_FORM	6
#define EVENT_SHOW_MENUBAR	7
#define EVENT_HIDE_MENUBAR	8

#define EVENT_TEXTBOX_SCROLLED	9
#define EVENT_IRDA_FRAME_RECEIVED	10

#define EVENT_USERDEF		1000	// the start number of user defined message

typedef struct {
	short x;
	short y;
	unsigned long jiffies;
	U16 event;
}S_MESSAGE;

#define MAX_MSG_COUNT 30
#define MAX_KEYMSG_COUNT 16


/********* the virtual key code of PDA **********/
#define KEY_POWER		1
#define KEY_RIGHT		2
#define KEY_UP			3
#define KEY_DOWN		4
#define KEY_ENTER		5
#define KEY_LEFT		6
#define KEY_RETURN	7
#define KEY_KEYBOARD	8
#define KEY_LIGHT		9
#define KEY_ADDRESS	10
#define KEY_CALCULATOR	11
#define KEY_CLOCK	12
#define KEY_TODO		13
#define KEY_SET_CONTRAST	14
#define KEY_HOTSYNC	15
#define KEY_FIND	16

/*************** User defined font routine structure **************/
typedef struct {
	short index;			// the font identify of the font library,for user-defined font, start from 16
	short x_offset,y_offset,width,height;
	short (*GetStringWidth)(char *);
	short (*GetFontHeight)(void);
	void (*DrawString)(int,int,char *,GraphicsContext *);
}FONT_ROUTINE;

#define FONT_SYS		0

#define FONT_SMALL		0
#define FONT_MEDIUM		1
#define FONT_LARGE		2


typedef struct {
	U8	id;
	U16	event;
	U32	lParam;
	U8	keycode;
	short	value;
}GUI_MSG;

/********* system references ***************/
#define VOL_MUTE	0
#define VOL_LOW	1
#define VOL_MEDIUM	2
#define VOL_HIGH	3

#define KB_NATURAL	0
#define KB_TRADITIONAL 1
typedef struct {
	short auto_off_time;	// auto-off time
	short sys_beep_vol;
	short alarm_beep_vol;
	short game_beep_vol;
	short kb_type;
}SystemPreference;



/************ Digitizer data format ***************

	Sometimes we wanna got raw data of digitizer ...

 **************************************************/
#define DATA_NORMAL	0
#define DATA_RAW		1

/******** function MsgBox related marcos *******/
#define MB_OK			0x01
#define MB_CANCEL	0x02
#define MB_OKCANCEL	(MB_OK|MB_CANCEL)
#define MB_YES			0x04
#define MB_NO			0x08
#define MB_YESNO		(MB_YES|MB_NO)

/******** Fixed writable directory *********/
#define PATH_INI	"//var//ini"

/******** Handheld types *********/
#define HANDHELD_EMULATOR	0
#define HANDHELD_PROCOM	1
#define HANDHELD_PALM3X	2
#define HANDHELD_PALMVX	3


#include "app_info.h"

#endif /* __LINUX_GUI_H */