#ifndef _GUI_INTERNAL_H
#define _GUI_INTERNAL_H

#include "linux/gui.h"

/*** control byte for PDA device peripherals ***/
#define CMD_SET_BACKLIGHT	0x01
#define CMD_GET_BATTERY		0x02
#define CMD_SET_CONTRAST		0x03
#define CMD_WRITE_SPEAKER	0x04
#define CMD_POWER_OFF		0x05
#define CMD_SET_AUTO_OFF	0x07
#define CMD_GET_AUTO_OFF	0x08
#define CMD_SUSPEND_MODE	0x09

typedef enum {
	CMD_CLIPRECT,
	CMD_SET_AVAILABLE_REGION,
	CMD_XOR_DRAW_LINE,		// xor_draw_line
	CMD_DRAW_LINE,				// draw_line
	CMD_DRAW_ELLIPSE,			// draw_ellipse
	CMD_DRAW_PIXEL,				// draw_pixel
	CMD_DRAW_RECT,				// draw_rect
	CMD_BITBLT,						// bitblt
	CMD_PATTERNFILL,				// patternfill
	CMD_CREATEFORM,				// CreateForm
	CMD_REFRESHCOMPONENT,	// RefreshComponent
	CMD_SETSPINBUTTONTEXT,	// SetSpinButtonText
	CMD_REFRESHMULTILINESTRING,	// RefreshMultiLineString
	CMD_REFRESHCOMBOBOXSTRING,		// RefreshComboBoxString
	CMD_HIDECOMPONENT,			// HideComponent
	CMD_CLOSEFORM,					// CloseForm
	CMD_SETFONT,						// SetFont	
	CMD_GETFONTHEIGHT,				// GetFontHeight
	CMD_GETSTRINGWIDTHINCLUDESPACING,	// GetStringWidthIncludeSpacing
	CMD_GETSTRINGWIDTH,				// GetStringWidth
	CMD_DRAWSTRINGINRECT,			// DrawStringInRect
	CMD_DO_DRAWSTRING,					// do_drawString
	CMD_DRAWSTRING,						// DrawString
	CMD_GETTEXTBOXLINES,				// GetTextBoxLines
	CMD_SETTEXTBOXCURSORPOS,		// SetTextBoxCursorPos
	CMD_REFRESHTEXTBOXCONTENTS,	// RefreshTextBoxContents	
	CMD_GETCURRENTACTIVATEDTEXTBOX,	// GetCurrentActivatedTextBox
	CMD_SETTEXT,										// set_text
	CMD_CLOSECURSOR,								// CloseCursor
	CMD_BLINKCURSOR,								// BlinkCursor
	CMD_SETFOCUS,									// SetFocus
	CMD_GETMESSAGE,								// GetMessage
	CMD_SHOWSOFTKEY,								// ShowSoftKey
	CMD_GUI_HOW_TO,								// gui_how_to
	CMD_GUI_ABOUT,									// gui_about
	
	/**** the following is the temperary functions ******/
	CMD_DRAW_VSCROLLBUTTONS,	// draw_vscrollbuttons
	CMD_DRAW_CHECKBUTTON,		// draw_checkbutton
	CMD_DRAWMENU,						// draw_menu
	CMD_HIDE_MENUBAR,					// hide_menubar
	CMD_DEALWITH_TEXTBOX				// dealwith_textbox
		
}CMD_GUI;

/**************************************************
 *
 *	Exported function argument descriptions
 *
 **************************************************/

// xor_draw_line(x1,y1,x2,y2);
typedef struct {
	short x1,y1,x2,y2
}ARG_XOR_DRAW_LINE;

// draw_line(x1,y1,x2,y2,color)
typedef struct {
	short x1,y1,x2,y2,color
}ARG_DRAW_LINE;

// draw_ellipse(x,y,a,b,*gc)
typedef struct {
	short x,y,a,b;
	GraphicsContext *gc;
}ARG_DRAW_ELLIPSE;

// draw_pixel(x,y,color)
typedef struct {
	short x,y;
	U8 color;
}ARG_DRAW_PIXEL;

// draw_rect(r,*gc)
typedef struct {
	RECT r;
	GraphicsContext *gc;
}ARG_DRAW_RECT;

// bitblt(src_x,src_y,w,h,dest_x,dest_y,*src,src_unit_per_line,*dest,dest_unit_per_line,*gc)
typedef struct {
	short src_x,src_y,w,h,dest_x,dest_y;
	U8 *src;
	short src_units_per_line;
	U8 *dest;
	short dest_units_per_line;
	GraphicsContext *gc;
}ARG_BITBLT;

// patternfill(dest_x,dest_y,w,h,*dest,dest_units_per_line,*gc)
typedef struct {
	short dest_x,dest_y,w,h;
	U8 *dest;
	short dest_units_per_line;
	GraphicsContext *gc;
}ARG_PATTERNFILL;

// RefreshComponent(idx,*dlg)
typedef struct {
	short idx;
	FORM *dlg;
}ARG_REFRESHCOMPONENT;

// SetSpinButtonText(*dlg,*str)
typedef struct {
	FORM *dlg;
	char *str
}ARG_SETSPINBUTTONTEXT;

// RefreshMultiLineString(*dlg,flag)
typedef struct  {
	FORM *dlg;
	short flag;
}ARG_REFRESHMULTILINESTRING;

// HideComponent(idx,*dlg)
typedef struct {
	short idx;
	FORM *dlg;
}ARG_HIDECOMPONENT;

// draw_checkbutton
typedef struct {
	FORM *dlg;
	short value;
}ARG_DRAW_CHECKBUTTON;

// GetStringWidthIncludeSpacing
typedef struct {
	char *str;
	short font_lib;
}ARG_GETSTRINGWIDTHINCLUDESPACING;

// GetStringWidth
typedef struct {
	char *str;
	short font_lib;
}ARG_GETSTRINGWIDTH;

// DrawStringInRect
typedef struct {
	RECT *r;
	char *data;
	GraphicsContext *gc;
	short font_lib,align_flag;
}ARG_DRAWSTRINGINRECT;

// do_drawString
typedef struct {
	short x,y;
	char *data;
	GraphicsContext *gc;
	short font_lib;
}ARG_DO_DRAWSTRING;

// DrawString
typedef struct {
	short x,y;
	char *str;
	GraphicsContext *gc;
}ARG_DRAWSTRING;

// GetTextBoxLines
typedef struct {
	FORM *dlg;
	short ret_value;
}ARG_GETTEXTBOXLINES;

// SetTextBoxCursorPos
typedef struct {
	FORM *dlg;
	short pos;
}ARG_SETTEXTBOXCURSORPOS;

// dealwith_textbox
typedef struct {
	FORM *dlg;
	PIXEL px;
}ARG_DEALWITH_TEXTBOX;

// RefreshTextBoxContnets
typedef struct {
	FORM *dlg;
	short start_pos,end_pos,flag
}ARG_REFRESHTEXTBOXCONTENTS;
	
// set_text
typedef struct {
	FORM *dlg;
	short code;
}ARG_SETTEXT;

// SetFocus
typedef struct {
	FORM *dlg;
	short id;
}ARG_SETFOCUS;

// GetMessage(*dlg,*msg)
typedef struct {
	FORM *dlg;
	GUI_MSG *msg;
}ARG_GETMESSAGE;

// ShowSoftKey(*str,short max_char)
typedef struct {
	char *str;
	short max_char;
}ARG_SHOWSOFTKEY;

// gui_how_to(*str)
typedef struct {
	char *str;
	short ret_value;
}ARG_GUI_HOW_TO;

// gui_about(*title,*version)
typedef struct {
	char *title;
	char *version;
	short ret_value;
}ARG_GUI_ABOUT;
#endif