#ifndef _MINI_DB_H
#define _MINI_DB_H


typedef int	DBHANDLE;
/***********************************************
 *
 *	General marcos of DB
 *
 ***********************************************/
#define MAX_FILENAME_LENGTH		16

/***********************************************
 *
 *	Definitions of field type of DB
 *
 ***********************************************/
#define DBCHAR		0x0001
#define DBINT		0x0002
#define DBLONG		0x0003
#define DBDATE		0x0004
#define DBTIME		0x0005
#define DBSTRING	0x0006
#define DBBINARY	0x0007
#define DBVARCHAR	0x0008


/***********************************************
 *
 *	Definitions of record property of DB
 *
 ***********************************************/
#define DB_RECORD_BUSY		0x01
#define DB_RECORD_SECURITY	0x02
#define DB_RECORD_DELETED	0x04
#define DB_RECORD_UPDATED	0x08
#define DB_CATEGORY_ALL		0x10
#define DB_SHOW_SECURITY	0x00
#define DB_HIDE_SECURITY	0x02


/***********************************************
 *
 *	Definitions of Error Numbers of DB API
 *
 ***********************************************/
#define DB_ERROR_OPENDATABASE		-1
#define DB_ERROR_CLOSEDATABASE		-1
#define DB_ERROR_READDATABASE		-1
#define DB_ERROR_WRITEDATABASE		-1
#define DB_ERROR_CREATEDATABASE		-1
#define DB_ERROR_DELETEDATABASE		-1
#define DB_ERROR_NOERROR			0
#define DB_ERROR_ADDCATEGORY		-1
#define DB_ERROR_DELETECATEGORY		-1
#define DB_ERROR_UPDATECATEGORY		-1
#define DB_ERROR_GETCATEGORY		-1
#define DB_ERROR_SETDBCATEGORY		-1
#define DB_ERROR_ALLOCATEMEMORY		-1
#define DB_ERROR_ADDRECORD		-1
#define DB_ERROR_DELETERECORD		-1
#define DB_ERROR_UPDATERECORD		-1
#define DB_ERROR_GETRECORD		-1


/***********************************************
 *
 *	CreateDatebase flags
 *
 ***********************************************/
#define DB_SET_OVERWRITE	0x10
//#define DB_SET_NO_FIELDHEAD	0x20
#define DB_BLOCK_NONE		0
#define DB_BLOCK_16			4
#define DB_BLOCK_32			5
#define DB_BLOCK_64			6
#define DB_BLOCK_128		7
#define DB_BLOCK_256		8
#define DB_BLOCK_512		9
#define DB_BLOCK_1024		10

/***********************************************
 *
 *	Definitions of database convert DB
 *
 ***********************************************/
#define DB_LITTLE_ENDIAN		0x3412
#define DB_BIG_ENDIAN		0x1234


typedef struct{
	char 	field_name[MAX_FILENAME_LENGTH];
	short	field_length;
	short	field_type;
}DBFIELD;


/***********************************************
 *
 *	Open Datebase flags
 *
 ***********************************************/
#define DB_READ_ONLY	1
#define DB_EXCLUSIVE	2
//#define DB_APPEND_DATA	4
#define DB_PURGE_DATA	5

/***********************************************
 *
 *	Functions of DB
 *
 ***********************************************/

/************************************************************
 *
 *	Create a new database
 *
 *	Return Value:	-1: 		Create failed
 *				none zero:	handle of the table
 *
 ************************************************************/
int LDACreateDB(char *DBName,short DBFlags,DBFIELD *dbField,short dbFieldNumber);
int LDAAddDBCategory(DBHANDLE fdidx,char * cate_name);
int LDADeleteDBCategory(DBHANDLE fdidx,int cate_index);
int LDAUpdateDBCategory(DBHANDLE fdidx,int cate_index,char * cate_name);
int LDAGetDBCategoryTotal(DBHANDLE fdidx);
int LDAGetDBCategoryAlphbet(DBHANDLE fdidx,int cate_rank, char * cate_name);
DBHANDLE LDAOpenDB(char *DBName,short DBFlag);
int LDADeleteDB(char *DBName);
int LDACloseDB(DBHANDLE fdidx,short flag);
int LDASetDBCategory(DBHANDLE fdidx,char cate_option);
int LDASetDBSecurity(DBHANDLE fdidx,char secu_option);
long LDAGetRecordNumber(DBHANDLE fdidx);
long LDAGetTotalRecords(DBHANDLE fdidx);
int LDAAddRecord(DBHANDLE fdidx,void *data,char category,char property);
long LDAFindRecordByValue(DBHANDLE fdidx,void *data,void * CompareValue,char * category,char * property);
int LDAGetNextRecord(DBHANDLE fdidx,void *data,char * category,char * property);
int LDAGetRecord(DBHANDLE fdidx,void *data,long RecordId,char * category,char * property);
int LDADeleteRecord(DBHANDLE fdidx,short RecordId);
int LDAUpdateRecord(DBHANDLE fdidx,void *data,short RecordId,char category,char property);
int LDAConvertByte(char *DBName,short option);


#endif